///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 
// Copyright Nurve Networks LLC 2008
// 
// Filename: REM_AVR_NTSC_128_96_ULTRA_V010.h
//
// Original Author: Remi Veilleux
// 
// Last Modified: 01.09.09
//
// Description: Header file for REM_AVR_NTSC_128_96_ULTRA_V010.s
//
// Overview: 
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// watch for multiple inclusions
#ifndef REM_AVR_NTSC_128_96_ULTRA
#define REM_AVR_NTSC_128_96_ULTRA

// interrupt rates for NTSC/VGA
#define VGA_INTERRUPT_RATE   910
#define NTSC_INTERRUPT_RATE  1817
#define VIDEO_INTERRUPT_RATE (NTSC_INTERRUPT_RATE)

// with the current D/A converter range 0 .. 15, 1 = 62mV
#define NTSC_BLACK    (0x4F)     // 0.3V (about 30-40 IRE)

#define COLOR_LUMA    (0x80)

#define NTSC_GREEN    (COLOR_LUMA + 0)        
#define NTSC_YELLOW   (COLOR_LUMA + 2)        
#define NTSC_ORANGE   (COLOR_LUMA + 4)        
#define NTSC_RED      (COLOR_LUMA + 5)        
#define NTSC_VIOLET   (COLOR_LUMA + 9)        
#define NTSC_PURPLE   (COLOR_LUMA + 11)        
#define NTSC_BLUE     (COLOR_LUMA + 14)        

#define NTSC_GRAY0    (NTSC_BLACK + 0x10)        
#define NTSC_GRAY1    (NTSC_BLACK + 0x20)        
#define NTSC_GRAY2    (NTSC_BLACK + 0x30)        
#define NTSC_GRAY3    (NTSC_BLACK + 0x40)        
#define NTSC_GRAY4    (NTSC_BLACK + 0x50)        
#define NTSC_GRAY5    (NTSC_BLACK + 0x60)        
#define NTSC_GRAY6    (NTSC_BLACK + 0x70)        
#define NTSC_GRAY7    (NTSC_BLACK + 0x80)        
#define NTSC_GRAY8    (NTSC_BLACK + 0x90)        
#define NTSC_GRAY9    (NTSC_BLACK + 0xA0)        
#define NTSC_WHITE    (NTSC_BLACK + 0xB0)        

// width and height of screen 
#define NTSC_SCREEN_WIDTH     128 // NOTE: this define has absolutely no effect. The horizontal resolution is calculated in NTSC_Draw_Active_Line
#define NTSC_SCREEN_HEIGHT    192 // 96 pixels * 2 (output each scanline twice)

#define TILEMAP_WIDTH 17 // 1 extra tile for horizontal scrolling
#define TILEMAP_HEIGHT 15 // 3 extra row to allow vertical scrolling + status screen

#define MAXSPRITES 8 // WARNING: This is FIXED to 8 sprites: Assembler code will always render 8 sprites
                     // DO NOT CHANGE THIS.

// controls the number of lines to shift the display vertically, later assign to global variable, so caller can control
#define NTSC_VERTICAL_OFFSET  ((262-(NTSC_SCREEN_HEIGHT))/2)

// these indicate the start, end, of the entire video scan and of the active lines, always in terms of real video lines 0
// eg. 0..261 for NTSC, 0..479 for VGA
#define START_VIDEO_SCAN      0
#define START_ACTIVE_SCAN     (NTSC_VERTICAL_OFFSET)
#define END_ACTIVE_SCAN       (NTSC_SCREEN_HEIGHT + NTSC_VERTICAL_OFFSET+1)
#define END_VIDEO_SCAN        261

// simply interogate the video driver's curr_line variable and determine where the scanline is
#define VIDEO_ACTIVE(video_line)          ( ((video_line) >= START_ACTIVE_SCAN && (video_line) <= END_ACTIVE_SCAN) ? 1 : 0)
#define VIDEO_INACTIVE(video_line)        ( ((video_line) < START_ACTIVE_SCAN || (video_line) > END_ACTIVE_SCAN) ? 1 : 0)
#define VIDEO_TOP_OVERSCAN(video_line)    ( ((video_line) >= 0 && (video_line) < START_ACTIVE_SCAN) ? 1 : 0)
#define VIDEO_BOT_OVERSCAN(video_line)    ( ((video_line) > END_ACTIVE_SCAN) ? 1 : 0)

// number of clocks per pixel (8 in the case of NTSC)
#define CLOCKS_PER_PIXEL      8 

// end multiple inclusions 
#endif
