///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 
// Copyright Nurve Networks LLC 2008
// 
// Filename: REM_AVR_NTSC_DOOM_FLOOR_8_V011.h
//
// Original Author: Remi Veilleux
// 
// Last Modified: 02.12.09
//
// Description: Header file for REM_AVR_NTSC_DOOM_FLOOR_8_V011.s
//
// Modified : 
// New version with texture stored in Flash (instead of SRAM)
// With different image for ceiling and floor
// And 8 levels of precalculated mip-mapping / shading
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES ///////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// watch for multiple inclusions
#ifndef REM_AVR_NTSC_DOOM_FLOOR_8_V011
#define REM_AVR_NTSC_DOOM_FLOOR_8_V011

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MACROS /////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINES/////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// interrupt rates for NTSC/VGA
#define VGA_INTERRUPT_RATE   910
#define NTSC_INTERRUPT_RATE  1817
#define VIDEO_INTERRUPT_RATE (NTSC_INTERRUPT_RATE)

// with the current D/A converter range 0 .. 15, 1 = 62mV
#define NTSC_BLACK    (0x4F)     // 0.3V (about 30-40 IRE)

#define COLOR_LUMA    (0x80)

#define NTSC_GREEN    (COLOR_LUMA + 0)        
#define NTSC_YELLOW   (COLOR_LUMA + 2)        
#define NTSC_ORANGE   (COLOR_LUMA + 4)        
#define NTSC_RED      (COLOR_LUMA + 5)        
#define NTSC_VIOLET   (COLOR_LUMA + 9)        
#define NTSC_PURPLE   (COLOR_LUMA + 11)        
#define NTSC_BLUE     (COLOR_LUMA + 14)        

#define NTSC_GRAY0    (NTSC_BLACK + 0x10)        
#define NTSC_GRAY1    (NTSC_BLACK + 0x20)        
#define NTSC_GRAY2    (NTSC_BLACK + 0x30)        
#define NTSC_GRAY3    (NTSC_BLACK + 0x40)        
#define NTSC_GRAY4    (NTSC_BLACK + 0x50)        
#define NTSC_GRAY5    (NTSC_BLACK + 0x60)        
#define NTSC_GRAY6    (NTSC_BLACK + 0x70)        
#define NTSC_GRAY7    (NTSC_BLACK + 0x80)        
#define NTSC_GRAY8    (NTSC_BLACK + 0x90)        
#define NTSC_GRAY9    (NTSC_BLACK + 0xA0)        
#define NTSC_WHITE    (NTSC_BLACK + 0xB0)        

// width and height of screen
#define NTSC_SCREEN_WIDTH	 82
#define NTSC_SCREEN_HEIGHT    206

// controls the number of lines to shift the display vertically, later assign to global variable, so caller can control
#define NTSC_VERTICAL_OFFSET  ((262-(NTSC_SCREEN_HEIGHT))/2)

// these indicate the start, end, of the entire video scan and of the active lines, always in terms of real video lines 0
// eg. 0..261 for NTSC, 0..479 for VGA
#define START_VIDEO_SCAN      0
#define START_ACTIVE_SCAN     (NTSC_VERTICAL_OFFSET)
#define END_ACTIVE_SCAN       (NTSC_SCREEN_HEIGHT + NTSC_VERTICAL_OFFSET)
#define END_VIDEO_SCAN        261

// number of clocks per pixel (8 in the case of NTSC)
#define CLOCKS_PER_PIXEL      8 

// end multiple inclusions 
#endif
