/* db_vmimage.c - bytecode image file routines
 *
 * Copyright (c) 2009 by David Michael Betz.  All rights reserved.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include "db_vm.h"
#include "../common/db_packet.h"

/* LoadImageFile - load an image file */
int LoadImageFile(Interpreter *i, char *name)
{
    uint8_t buf[PKTMAXLEN];
    uint16_t off, cnt;
    FileHdr hdr;
    VMFILE *fp;

    /* open the image file */
    if (!(fp = VM_fopen(name, "rb")))
        return VMFALSE;

    /* read the image header */
    if (VM_fread(&hdr, sizeof(FileHdr), fp) != sizeof(FileHdr))
        goto fail;

	/* start the image load */
	if (!StartImageLoad(i, &hdr))
		goto fail;

    /* read the code */
    VM_fseek(fp, hdr.codeOffset, SEEK_SET);
    for (off = 0; off < hdr.codeSize; off += cnt) {

        /* determine how much initializer data to read */
        if ((cnt = hdr.codeSize - off) > sizeof(buf))
			cnt = sizeof(buf);

        /* read the next block of code */
        if (VM_fread(buf, cnt, fp) != cnt)
            goto fail2;

        /* load the code */
        VM_LoadCode(off, buf, cnt);
    }

    /* read the variable initializers */
    VM_fseek(fp, hdr.initOffset, SEEK_SET);
    for (off = 0; off < hdr.initSize; off += cnt) {

        /* determine how much initializer data to read */
        if ((cnt = hdr.initSize - off) > sizeof(buf))
			cnt = sizeof(buf);

        /* read the next block of initializer data */
        if (VM_fread(buf, cnt, fp) != cnt)
            goto fail2;

        /* load the initialization data */
		VM_LoadInit(off, buf, cnt);

        /* process the initializer data */
        ProcessInitData(i, buf, cnt);
    }

    /* close the image file */
    VM_fclose(fp);

    /* finish the image load */
    VM_StoreMemoryImageHeader(i);
    FinishImageLoad(i);

    /* return successfully */
    return VMTRUE;

    /* cleanup and report a failure */
fail2:
    VM_FreeImage(i);
fail:
    VM_fclose(fp);
    return VMFALSE;
}
