/* listener.h - definitions for the interactive text window
 *
 * Copyright (c) 2009 by David Michael Betz.  All rights reserved.
 *
 */

#ifndef __LISTENER_H__
#define __LISTENER_H__

#include "wx/wx.h"
#include "wx/textctrl.h"

class ListenerCtrl;

class Listener : public wxTextCtrl
{
public:

    Listener(
        wxWindow *parent,
        wxWindowID id,
        const wxString &value,
        const wxPoint &pos = wxDefaultPosition,
        const wxSize &size = wxDefaultSize,
        int style = 0);
    void Printf(wxString fmt,...);
    void PutS(wxString str);
    void PutC(wxChar ch);
    void FlushOutput();
    void SetInputHandler(void (*handler)(void *data, const char *buf), void *data);

    void OnChar(wxKeyEvent &event);

private:

    wxString m_outBuf;
    wxFont m_font;
    wxTextAttr m_inStyle;
    wxTextAttr m_outStyle;
    void (*m_inputHandler)(void *data, const char *buf);
    void *m_inputHandlerData;

    void Append(wxString str);
    void SetInputStyle();
    void SetOutputStyle();
    bool CheckForEOL();
    void HandleUserInput();
    void FindInputText(long current, long *pStart, long *pEnd);

    DECLARE_EVENT_TABLE()
};

#endif
