/* xgsbasic.h - definitions for the wxWidgets xgsbasic gui
 *
 * Copyright (c) 2009 by David Michael Betz.  All rights reserved.
 *
 */

#ifndef __XGSBASICW_H__
#define __XGSBASICW_H__

#include "wx/wx.h"
#include "wx/config.h"

#include "main.h"
#include "listener.h"

extern "C" {    
    #include "../runtime/db_vm.h"
}

enum {
    ID_DOWNLOAD_CARD = wxID_HIGHEST + 1,
    ID_DOWNLOAD_FLASH,
    ID_DOWNLOAD_SRAM,
    ID_PORT_COM1,
    ID_PORT_COM2,
    ID_PORT_COM3,
    ID_PORT_COM4,
    ID_PORT_COM5,
    ID_PORT_COM6,
    ID_LOAD,
    ID_RELOAD
};

enum {
    kDownloadCard,
    kDownloadFlash,
    kDownloadSRAM
};

class Basic : public wxApp
{
public:

    void ConsolePutC(int ch);
    void ConsoleFlushOutput();
    void ConsoleGetLine();

    void Warning(char *fmt, ...);

    void SaveSettingsAndExit();

    void OpenPort(const char *port, bool init);
    void ClosePort();

    int DownloadMode() { return m_downloadMode; }
    void SetDownloadMode(int mode) { m_downloadMode = mode; }

    void CompileAndLoad();
    void ReloadLastFile();

    virtual bool OnInit();

private:

    void SizeAndPositionWindows();
    void SaveWindowPositions();
    void SaveWindowPosition(wxString key,wxTopLevelWindow *win);
    void RestoreSettings();
    void RestoreWindowPositions();
    void RestoreWindowPosition(wxString key,wxTopLevelWindow *win);

    static void HandleInput(void *data, const char *buf);

    void Load(wxString file);
    void Download(wxString file);
    
    wxConfig *m_config;
    wxRect m_screenRect;
    MainFrame *m_main;

    long m_downloadMode;

    bool m_portOpen;
    wxString m_port;

    wxString m_lastFileLoaded;
};

DECLARE_APP(Basic)

#endif
