///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 
// Copyright Nurve Networks LLC 2008
// 
// Filename: XGS_AVR_NTSC_TILE_MODE2_V010.h
//
// Original Author: Andre' LaMothe
// 
// Last Modified: 9.1.08
//
// Description: Header file for XGS_AVR_NTSC_TILE_MODE2_V010.s
//
// Overview: 
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES ///////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// watch for multiple inclusions
#ifndef XGS_AVR_NTSC_TILE_MODE2
#define XGS_AVR_NTSC_TILE_MODE2

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MACROS /////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINES/////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// interrupt rates for NTSC/VGA
#define VGA_INTERRUPT_RATE   910
#define NTSC_INTERRUPT_RATE  1817
#define VIDEO_INTERRUPT_RATE (NTSC_INTERRUPT_RATE)

// with the current D/A converter range 0 .. 15, 1 = 62mV
#define NTSC_SYNC           (0x0F)     // 0V  
#define NTSC_BLACK          (0x4F)     // 0.3V (about 30-40 IRE)

#define NTSC_CBURST0        (0x40) 
#define NTSC_CBURST1        (0x41) 
#define NTSC_CBURST2        (0x42) 

#define COLOR_LUMA    (NTSC_CBURST0 + 0x40)

#define NTSC_GREEN    (COLOR_LUMA + 0)        
#define NTSC_YELLOW   (COLOR_LUMA + 2)        
#define NTSC_ORANGE   (COLOR_LUMA + 4)        
#define NTSC_RED      (COLOR_LUMA + 5)        
#define NTSC_VIOLET   (COLOR_LUMA + 9)        
#define NTSC_PURPLE   (COLOR_LUMA + 11)        
#define NTSC_BLUE     (COLOR_LUMA + 14)        

#define NTSC_GRAY0    (NTSC_BLACK + 0x10)        
#define NTSC_GRAY1    (NTSC_BLACK + 0x20)        
#define NTSC_GRAY2    (NTSC_BLACK + 0x30)        
#define NTSC_GRAY3    (NTSC_BLACK + 0x40)        
#define NTSC_GRAY4    (NTSC_BLACK + 0x50)        
#define NTSC_GRAY5    (NTSC_BLACK + 0x60)        
#define NTSC_GRAY6    (NTSC_BLACK + 0x70)        
#define NTSC_GRAY7    (NTSC_BLACK + 0x80)        
#define NTSC_GRAY8    (NTSC_BLACK + 0x90)        
#define NTSC_GRAY9    (NTSC_BLACK + 0xA0)        
#define NTSC_WHITE    (NTSC_BLACK + 0xB0)        


// width and height of screen 
#define NTSC_SCREEN_WIDTH     180
#define NTSC_SCREEN_HEIGHT    192
#define SCREEN_WIDTH          180
#define SCREEN_HEIGHT         192


// controls the number of lines/pixels to shift the display vertically/horizontally, 
// later assign to global variable, so caller can control
#define NTSC_VERTICAL_OFFSET   36
#define NTSC_HORIZONTAL_OFFSET 80

#define VERTICAL_OFFSET        36
#define HORIZONTAL_OFFSET      80

// these indicate the start, end, of the entire video scan and of the active lines, always in terms of real video lines 0
// eg. 0..261 for NTSC, 0..479 for VGA
#define START_VIDEO_SCAN      0
#define START_ACTIVE_SCAN     (VERTICAL_OFFSET)
#define END_ACTIVE_SCAN       (SCREEN_HEIGHT + VERTICAL_OFFSET+1)
#define END_VIDEO_SCAN        261

// simply interogate the video driver's curr_line variable and determine where the scanline is
#define VIDEO_ACTIVE(video_line)          ( ((video_line) >= START_ACTIVE_SCAN && (video_line) <= END_ACTIVE_SCAN) ? 1 : 0)
#define VIDEO_INACTIVE(video_line)        ( ((video_line) < START_ACTIVE_SCAN || (video_line) > END_ACTIVE_SCAN) ? 1 : 0)
#define VIDEO_TOP_OVERSCAN(video_line)    ( ((video_line) >= 0 && (video_line) < START_ACTIVE_SCAN) ? 1 : 0)
#define VIDEO_BOT_OVERSCAN(video_line)    ( ((video_line) > END_ACTIVE_SCAN) ? 1 : 0)

// number of bits each pixel is encoded with
#define NTSC_PIXELS_PER_BYTE   4            
#define PIXELS_PER_BYTE        4            

// number of clocks per pixel (8 in the case of NTSC)
#define CLOCKS_PER_PIXEL       8 

// tile engine defines
#define TILE_MAP_WIDTH         22 
#define TILE_MAP_HEIGHT        24 

// tile engine bitmap constants
#define TILE_BITMAP_WIDTH      8   // width and height of tile bitmap  
#define TILE_BITMAP_HEIGHT     8
#define TILE_BITMAP_SIZE       64  // size of tile bitmap       

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// TYPES/CLASSES //////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GLOBALS ////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// EXTERNALS //////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// PROTOTYPES /////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// end multiple inclusions 
#endif
