/* db_vmfcnavr.c - intrinsic functions
 *
 * Copyright (c) 2009 by David Michael Betz.  All rights reserved.
 *
 */

#include <stdlib.h>
#include <avr/sleep.h>
#include "../runtime/db_vm.h"

#include "XGS_AVR_SYSTEM_V010.h"
#include "XGS_AVR_GFX_LIB_V010.h"
#include "XGS_AVR_GAMEPAD_DRV_V010.h"
#include "XGS_AVR_SOUND_DRV_V010.h"
#include "XGS_AVR_NTSC_TILE_MODE2_V010.h"

/* tiles */
static unsigned char *myTiles = NULL;
static int myTileCount = 0;

/* tile map */
static unsigned char *myTileMap = NULL;

/* sprite table */
//volatile SPRITE sprite_table[NUM_SPRITES];

/* graphics state */
static int graphicsStarted = VMFALSE;

void VM_fcninit(void)
{
    myTiles = NULL;
    myTileMap = NULL;
    graphicsStarted = VMFALSE;
    memset(tile_init_start, 0, tile_init_end - tile_init_start);
    NTSC_Init();
}

void VM_fcnstop(void)
{
    GFX_Stop();
}

void fcn_waitforvsync(Interpreter *i)
{
	do {
        sleep_cpu();
    } while (curr_line != END_ACTIVE_SCAN);
    Drop(i, 1);
}

void fcn_gamepad(Interpreter *i)
{
	int which = i->sp[0];
	i->sp[-1] = Gamepad_Read(which == 0 ? GAMEPAD0 : GAMEPAD1);
	Drop(i, 1);
}

void fcn_sound(Interpreter *i)
{
	int freq = i->sp[0];
	Sound_Play(freq);
	Drop(i, 2);
}

uint8_t *g_VRAMBuffer;

void fcn_plot(Interpreter *i)
{
	int x = i->sp[-2];
	int y = i->sp[-1];
	int color = i->sp[0];
	GFX_Plot(x, y, color, g_VRAMBuffer);
	Drop(i, 4);
}

void fcn_line(Interpreter *i)
{
	int x1 = i->sp[-4];
	int y1 = i->sp[-3];
	int x2 = i->sp[-2];
	int y2 = i->sp[-1];
	int color = i->sp[0];
	GFX_Draw_Line(x1, y1, x2, y2, color, g_VRAMBuffer);
	Drop(i, 6);
}

void fcn_hline(Interpreter *i)
{
	int x1 = i->sp[-3];
	int x2 = i->sp[-2];
	int y = i->sp[-1];
	int color = i->sp[0];
	GFX_Draw_HLine(x1, x2, y, color, g_VRAMBuffer);
	Drop(i, 5);
}

void fcn_vline(Interpreter *i)
{
	int x = i->sp[-3];
	int y1 = i->sp[-2];
	int y2 = i->sp[-1];
	int color = i->sp[0];
	GFX_Draw_VLine(x, y1, y2, color, g_VRAMBuffer);
	Drop(i, 5);
}

void fcn_clear(Interpreter *i)
{
	int color = i->sp[0];
    GFX_Fill_Screen(color, g_VRAMBuffer);
	Drop(i, 2);
}

void fcn_fillrect(Interpreter *i)
{
#if 0
	int x = i->sp[-4];
	int y = i->sp[-3];
	int w = i->sp[-2];
	int h = i->sp[-1];
	int color = i->sp[0];
	GFX_FillRegion(x, y, w, h, color, g_VRAMBuffer);
#endif
	Drop(i, 6);
}

void fcn_circle(Interpreter *i)
{
	int x = i->sp[-3];
	int y = i->sp[-2];
	int r = i->sp[-1];
	int color = i->sp[0];
	GFX_Draw_Circle(x, y, r, color, g_VRAMBuffer);
	Drop(i, 5);
}

void fcn_position(Interpreter *i)
{
    int x = i->sp[-1];
    int y = i->sp[0];
	printf("position (%d, %d)\n", x, y);
	Drop(i, 3);
}

void fcn_text(Interpreter *i)
{
#if 0
    int x = i->sp[-2];
    int y = i->sp[-1];
	char *str = (char *)i->code + i->sp[0];
	DrawBitmapFontString(str, x, y, g_VRAMBuffer);
#endif
	Drop(i, 4);
}

/* start graphics when all of the tables have been setup */
static void TryStartGraphics(void)
{
    if (myTileMap && myTiles) {
#if 0
        int i;

        for (i = 0; i < NUM_SPRITES; ++i)
            sprite_table[i].state = 0;
#endif

        tile_map_ptr = myTileMap;
        tile_bitmaps_ptr = myTiles;

        g_clip_min_x = 0;
        g_clip_max_x = SCREEN_WIDTH - 1;
        g_clip_min_y = 0;
        g_clip_max_y = SCREEN_HEIGHT - 1;
        g_screen_width  = SCREEN_WIDTH;
        g_screen_height = SCREEN_HEIGHT;
        g_tile_map_width  = TILE_MAP_WIDTH;
        g_tile_map_height = TILE_MAP_HEIGHT;
        tile_map_logical_width = TILE_MAP_WIDTH;

        if (!graphicsStarted) {
            GFX_Init(VIDEO_INTERRUPT_RATE);
            set_sleep_mode(SLEEP_MODE_IDLE);
            sleep_enable();
            graphicsStarted = VMTRUE;
        }
    }
}

void fcn_settilemap(Interpreter *i)
{
	int16_t *array = &i->variables[i->sp[0]];
    int16_t dim1 = array[-2];
    int16_t dim2 = array[-1] / dim1;
    if (dim1 >= TILE_MAP_WIDTH && dim2 >= TILE_MAP_HEIGHT) {
        myTileMap = (unsigned char *)array;
        TryStartGraphics();
    }
	Drop(i, 2);
}

void fcn_settiles(Interpreter *i)
{
	int16_t *array = &i->variables[i->sp[0]];
    int16_t dim1 = array[-2];
    int16_t dim2 = array[-1] / dim1;
    int nTiles = dim2 / TILE_BITMAP_HEIGHT;
    if (dim1 == TILE_BITMAP_WIDTH && nTiles > 0 && (dim2 % TILE_BITMAP_HEIGHT) == 0) {
        myTiles = (unsigned char *)array;
        myTileCount = nTiles;
        TryStartGraphics();
    }
	Drop(i, 2);
}

void fcn_setcolortable(Interpreter *i)
{
	int16_t *array = &i->variables[i->sp[0]];
    int16_t dim1 = array[-2];
    int16_t dim2 = array[-1] / dim1;
    VM_printf("setcolortable (%d,%d)\n", dim1, dim2);
	Drop(i, 2);
}

void fcn_setsprite(Interpreter *i)
{
#if 0
    int16_t n = i->sp[-4];
    int16_t state = i->sp[-3];
    int16_t x = i->sp[-2];
    int16_t y = i->sp[-1];
    int16_t tile = i->sp[0];
    if (n >= 0 && n < NUM_SPRITES && myTiles && tile >= 0 && tile < myTileCount) {
        volatile SPRITE *sprite = &sprite_table[n];
        sprite->state = state;
        sprite->attr = 0;
        sprite->height = 0;
        sprite->x = x;
        sprite->y = y;
        sprite->bitmap = &myTiles[tile];
        TryStartGraphics();
    }
#endif
    Drop(i, 6);
}

