///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 
// Copyright Nurve Networks LLC 2008
// 
// Filename: RPE_NTSC_BMP_80_60_4_V010.h
//
// Original Author: Andre' LaMothe
// 
// Last Modified: 12.22.08
//
// Description: Header file for RPE_NTSC_BMP_80_60_4_V010.s
//
// Overview: 
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Modifications|Bugs|Additions:
//
// Author Name: Roy Eltham
//
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES ///////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// watch for multiple inclusions
#ifndef RPE_NTSC_BMP_80_60_4
#define RPE_NTSC_BMP_80_60_4

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MACROS /////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINES/////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// interrupt rates for NTSC/VGA
#define VGA_INTERRUPT_RATE   910
#define NTSC_INTERRUPT_RATE  1817
#define VIDEO_INTERRUPT_RATE (NTSC_INTERRUPT_RATE)

// with the current D/A converter range 0 .. 15, 1 = 62mV
#define NTSC_SYNC     (0x0F)     // 0V  
#define NTSC_BLACK    (0x4F)     // 0.3V (about 30-40 IRE)

#define NTSC_CBURST0  (0x40) 
#define NTSC_CBURST1  (0x41) 
#define NTSC_CBURST2  (0x42) 

#define COLOR_LUMA    (NTSC_CBURST0 + 0x40)

#define NTSC_GREEN     (COLOR_LUMA + 0)        
#define NTSC_YELLOW    (COLOR_LUMA + 1)        
#define NTSC_BROWN     (COLOR_LUMA + 2)        
#define NTSC_ORANGE    (COLOR_LUMA + 3)        
#define NTSC_RED       (COLOR_LUMA + 4)        
#define NTSC_CRIMSON   (COLOR_LUMA + 5)        
#define NTSC_MAGENTA   (COLOR_LUMA + 6)        
#define NTSC_DARKPINK  (COLOR_LUMA + 7)        
#define NTSC_PURPLE    (COLOR_LUMA + 8)        
#define NTSC_VIOLET    (COLOR_LUMA + 9)        
#define NTSC_SLATEBLUE (COLOR_LUMA + 10)        
#define NTSC_ROYALBLUE (COLOR_LUMA + 11)        
#define NTSC_BLUE      (COLOR_LUMA + 12)        
#define NTSC_COBALT    (COLOR_LUMA + 13)        
#define NTSC_SKYBLUE   (COLOR_LUMA + 14)        

#define NTSC_GRAY0    (NTSC_BLACK + 0x10)        
#define NTSC_GRAY1    (NTSC_BLACK + 0x20)        
#define NTSC_GRAY2    (NTSC_BLACK + 0x30)        
#define NTSC_GRAY3    (NTSC_BLACK + 0x40)        
#define NTSC_GRAY4    (NTSC_BLACK + 0x50)        
#define NTSC_GRAY5    (NTSC_BLACK + 0x60)        
#define NTSC_GRAY6    (NTSC_BLACK + 0x70)        
#define NTSC_GRAY7    (NTSC_BLACK + 0x80)        
#define NTSC_GRAY8    (NTSC_BLACK + 0x90)        
#define NTSC_GRAY9    (NTSC_BLACK + 0xA0)        
#define NTSC_WHITE    (NTSC_BLACK + 0xB0)        

// width and height of screen 
#define NTSC_SCREEN_WIDTH     80
#define NTSC_SCREEN_HEIGHT    60

#define SCREEN_WIDTH          80
#define SCREEN_HEIGHT         60

// controls the number of lines to shift the display vertically, later assign to global variable, so caller can control
#define NTSC_VERTICAL_OFFSET  14
#define VERTICAL_OFFSET       14

// these indicate the start, end, of the entire video scan and of the active lines, always in terms of real video lines 0
// eg. 0..261 for NTSC, 0..479 for VGA
#define START_VIDEO_SCAN      0
#define START_ACTIVE_SCAN     (VERTICAL_OFFSET)
#define END_ACTIVE_SCAN       (SCREEN_HEIGHT*4 + VERTICAL_OFFSET+1)
#define END_VIDEO_SCAN        261

// simply interogate the video driver's curr_line variable and determine where the scanline is
#define VIDEO_ACTIVE(video_line)          ( ((video_line) >= START_ACTIVE_SCAN && (video_line) <= END_ACTIVE_SCAN) ? 1 : 0)
#define VIDEO_INACTIVE(video_line)        ( ((video_line) < START_ACTIVE_SCAN || (video_line) > END_ACTIVE_SCAN) ? 1 : 0)
#define VIDEO_TOP_OVERSCAN(video_line)    ( ((video_line) >= 0 && (video_line) < START_ACTIVE_SCAN) ? 1 : 0)
#define VIDEO_BOT_OVERSCAN(video_line)    ( ((video_line) > END_ACTIVE_SCAN) ? 1 : 0)

// number of bits each pixel is encoded with
#define NTSC_PIXELS_PER_BYTE  2            
#define PIXELS_PER_BYTE       2            

// number of clocks per pixel (8 in the case of NTSC)
#define CLOCKS_PER_PIXEL      8 

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// TYPES/CLASSES //////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GLOBALS ////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// EXTERNALS //////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// PROTOTYPES /////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// end multiple inclusions 
#endif
